"""Indexing system for VibeDB."""

from vibedb.indexes.manager import IndexManager
from vibedb.indexes.analyzer import SlowQueryAnalyzer, QueryTimer
from vibedb.indexes.suggestions import IndexSuggestionEngine, suggest_indexes_for_query

__all__ = [
    "IndexManager",
    "SlowQueryAnalyzer",
    "QueryTimer",
    "IndexSuggestionEngine",
    "suggest_indexes_for_query",
]
