"""
VibeDB - SQLite-based document database for vibe coders.

Zero configuration, instant gratification.
"""

from vibedb.__version__ import __version__
from vibedb.core.database import VibeDB
from vibedb.core.document import Document
from vibedb.query.dsl import Q
from vibedb.errors import (
    VibeDBError,
    ValidationError,
    SlowQueryWarning,
    IndexError,
    QueryError,
)

__all__ = [
    "__version__",
    "VibeDB",
    "Document",
    "Q",
    "VibeDBError",
    "ValidationError",
    "SlowQueryWarning",
    "IndexError",
    "QueryError",
]
